@@ -448,7 +448,7 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
         msn_object = None
         icon_url = None
         if len(command.arguments) > idx:
-            if command.arguments[idx] != '0':
+            if command.arguments[idx] not in ('0', '1'):
                 msn_object = papyon.p2p.MSNObject.parse(self._client,
                                urllib.unquote(command.arguments[idx]))
         idx += 1
@@ -733,9 +733,16 @@ class NotificationProtocol(BaseProtocol, gobject.GObject):
                 address_book.profile.display_name)
 
         contacts = address_book.contacts.group_by_domain()
+        mask = ~(profile.Membership.REVERSE | profile.Membership.PENDING)
+
+        for contact in address_book.contacts:
+            if (contact.memberships & mask & ~profile.Membership.FORWARD) == \
+                    (profile.Membership.ALLOW | profile.Membership.BLOCK):
+                logger.warning("Contact is on both Allow and Block list; " \
+                               "removing from Allow list (%s)" % contact.account)
+                contact._remove_membership(profile.Membership.ALLOW)
 
         payloads = ['<ml l="1">']
-        mask = ~(profile.Membership.REVERSE | profile.Membership.PENDING)
         for domain, contacts in contacts.iteritems():
             payloads[-1] += '<d n="%s">' % domain
             for contact in contacts:
